from django.apps import AppConfig

class DjangoSecurityQuestionsConfig(AppConfig):
	name = 'django_security_questions'
	def ready(self):
		from .models import SecurityQuestion, SecurityAnswer
		from .serializers import SecurityQuestionSerializer, SecurityAnswerSerializer



from django.urls import include, re_path

from .views import add, change

urlpatterns = [
    re_path(r'^questions/add/$', add, name='questions_add'),
    re_path(r'^questions/chagne/$/', change, name='questions_change'),
]
urlpatterns = patterns('',
    url(
        regex=r'^questions/add/$',
        view='add',
        name='questions_add'
    ),
    url(
        regex=r'^questions/change/$',
        view='change',
        name='questions_change'
    ),
)



from django.conf import settings

# Custom User support section ==============================================={{{
#
# The following code (from === to ===) was copied from django-userena at
# 2014/05/24 to solve issue #18.
# The code is protected with BSD License.
#
# Ref: https://github.com/bread-and-pepper/django-userena/blob/master/userena/utils.py#L165-L176
#
# ---------------------------------------------------------------------------{{{
# Copyright (c) 2010, Petar Radosevic
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of the author nor the names of other
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ---------------------------------------------------------------------------}}}
#
# Django 1.5 compatibility utilities, providing support for custom User models.
# Since get_user_model() causes a circular import if called when app models are
# being loaded, the user_model_label should be used when possible, with calls
# to get_user_model deferred to execution time

user_model_label = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User
    get_user_model = lambda: User




from django.views.generic.base import TemplateResponseMixin
from django.views.generic.edit import FormMixin, ProcessFormView


class QuestionsAddView(FormMixin, TemplateResponseMixin, ProcessFormView):
    """A view for security questions

    GET:
        Display a QuestionsFormset where each form has a ``question`` and ``answer``
        The same question can not be picked twice.

    POST:
        Associate the user with the selected ``question`` and inputted ``answer``
    """
    template_name = r'django_security_questions/questions_form.html'




from .models import SecurityQuestion, SecurityAnswer
from rest_framework import serializers

class SecurityQuestionSerializer(serializers.ModelSerializer):
    class Meta:
        model = SecurityQuestion
        fields = '__all__'


class SecurityAnswerSerializer(serializers.ModelSerializer):
    class Meta:
        model = SecurityAnswer
        fields = '__all__'




from django import forms
from django.conf import settings
from django.forms import models, ValidationError
from django.forms.formsets import formset_factory, BaseFormSet
from django.forms.models import modelformset_factory
from django.http import Http404
from django.utils.translation import gettext_lazy as _

from .models import SecurityAnswer


class BaseSecurityQuestionsRegisterFormSet(models.BaseModelFormSet):
    class Meta:
        model = SecurityAnswer

    def __init__(self, *args, **kwargs):
        self.num_questions = getattr(settings, "QUESTIONS_NUM_REGISTER")
        super(BaseSecurityQuestionsRegisterFormSet, self).__init__(*args, **kwargs)

    def clean(self):
        if any(self.errors):
            # Do not bother validating the formset unless each form is valid on its own
            return
        if self.management_form.cleaned_data["TOTAL_FORMS"] < self.num_questions:
            raise ValidationError(
                _("ManagementForm data is missing or has been tampered with"),
                code='management_form_missing',
            )
        completed = 0
        values = set()
        for cleaned_data in self.cleaned_data:
            # form has data and we are not deleting it.
            if cleaned_data and not cleaned_data.get("DELETE", False):
                completed += 1
            # security question is not the same
            if cleaned_data.get("question", None):
                value = cleaned_data["question"]
                if value in values:
                    raise ValidationError(
                        _("Must select different security questions"),
                        code='security_questions_same'
                    )
                values.add(value)

        if completed < self.num_questions:
            raise ValidationError(
                _("%(value)s security question and answer pairs are required"),
                code='security_questions_cnt',
                params={'value': self.num_questions}
            )
        self.validate_unique()


SecurityQuestionsRegisterFormSet = modelformset_factory(SecurityAnswer,
                                                        fields=("question", "answer"),
                                                        formset=BaseSecurityQuestionsRegisterFormSet,
                                                        extra=getattr(settings, "QUESTIONS_NUM_REGISTER"),
                                                        can_delete=False
                                                        )


class SecurityQuestionForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.sa_obj = kwargs.pop("security_answer_obj")
        super(SecurityQuestionForm, self).__init__(*args, **kwargs)
        self.fields["user_answer"] = forms.CharField(label=self.sa_obj.question.question,
                                                     initial="", required=True)
        self.fields["security_question"] = forms.IntegerField(initial=self.sa_obj.pk,
                                                              widget=forms.HiddenInput(attrs={"readonly": True}))

    def clean_user_answer(self):
        if (hasattr(self, "cleaned_data") and
                self.cleaned_data.get("user_answer", None) and
                not self.cleaned_data.get("DELETE", False)):
            if not self.sa_obj.check_answer(self.cleaned_data["user_answer"]):
                raise ValidationError(
                    _("Incorrect answer to security question"),
                    code='security_question_incorrect'
                )
        return self.cleaned_data["user_answer"]


class BaseSecurityQuestionResetFormSet(BaseFormSet):
    def __init__(self, *args, **kwargs):
        self.num_questions = int(getattr(settings, "QUESTIONS_NUM_RESET"))
        self.user = kwargs.pop("user")
        super(BaseSecurityQuestionResetFormSet, self).__init__(*args, **kwargs)
        if len(self.data) > 0:
            # If data is being passed to the form (likely a POST)
            q_pks = []
            if int(self.management_form.cleaned_data["TOTAL_FORMS"]) != self.num_questions:
                raise ValidationError(
                    _("ManagementForm data is missing or has been tampered with"),
                    code='management_form_missing'
                )
            for i in range(self.num_questions):
                field_name = "form-%s-security_question" % i
                try:
                    q_pks.append(int(self.data[field_name]))
                except KeyError:
                    raise ValidationError(
                        _("Missing security question(s) and answer(s)"),
                        code='security_questions_missing'
                    )
            qs = SecurityAnswer.objects.filter(pk__in=q_pks).all()
            for pk in q_pks:
                qs = sorted(qs, key=lambda x: x.pk == pk)
            self.sa_objs = qs
        else:
            # No data is being passed to teh form (likely a GET)
            qs = SecurityAnswer.objects.filter(user=self.user).order_by("?")
            self.sa_objs = qs[:self.num_questions]
        self.extra = len(self.sa_objs)
        if self.extra < self.num_questions:
            raise Http404(_("User does not have %s security questions") % self.num_questions)

        for form in self.forms:
            form.empty_permitted = False

    def _construct_form(self, index, **kwargs):
        # Passing a SecurityAnswer objects to each form
        kwargs["security_answer_obj"] = self.sa_objs[index]
        return super(BaseSecurityQuestionResetFormSet, self)._construct_form(index, **kwargs)

    def clean(self):
        if any(self.errors):
            # Provide a generic answer, and do not leak information about which questions are incorrect
            raise ValidationError(
                _("Incorrect answer to 1 or more security questions"),
                code='security_questions_incorrect'
            )

SecurityQuestionsResetFormSet = formset_factory(SecurityQuestionForm,
                                                formset=BaseSecurityQuestionResetFormSet,
                                                can_delete=False
                                                )



from django.contrib.auth import hashers
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_bleach.models import BleachField

from .compat import user_model_label


class SecurityQuestion(models.Model):
    id = models.AutoField(primary_key=True)
    question = BleachField(max_length=150, null=False, blank=False)

    class Meta:
        db_table = "security_questions"

    def __str__(self):
        return f"{self.question}"

class SecurityAnswer(models.Model):
    user = models.ForeignKey(user_model_label, on_delete=models.CASCADE, related_name='user_security_answers', default=1)
    question = models.ForeignKey("SecurityQuestion", verbose_name=_("Security Question"), on_delete=models.CASCADE)
    answer = BleachField(max_length=100, null=False, blank=False)

    class Meta:
        db_table = "security_answer"
        unique_together = ('user', 'question')

    def __str__(self):
        return f"{self.user} - {self.question}"

    def hash_current_answer(self):
        """Rehashes the currently stored answer."""
        self.set_answer(self.answer)

    def set_answer(self, raw_answer):
        """Hashes and sets the answer."""
        if not bool(getattr(settings, "QUESTIONS_CASE_SENSITIVE", False)):
            raw_answer = raw_answer.upper()
        self.answer = hashers.make_password(raw_answer)

    def check_answer(self, raw_answer):
        """Checks if the raw answer matches the stored answer."""
        if not bool(getattr(settings, "QUESTIONS_CASE_SENSITIVE", False)):
            raw_answer = raw_answer.upper()

        def setter(raw_answer):
            if not hashers.check_password(raw_answer, self.answer)
                self.set_answer(raw_answer)
                self.save(update_fields=["answer"])

        return hashers.check_password(raw_answer, self.answer, setter)

    def set_unusable_answer(self):
        """Sets an answer that is no longer usable."""
        self.answer = hashers.make_password(None)

    def has_usable_answer(self):
        """Checks if the current answer is usable (not set to an unusable state)."""
        return hashers.is_password_usable(self.answer)




# Generated by Django 5.1.1 on 2024-11-13 10:08

import django.db.models.deletion
import django_bleach.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SecurityQuestion',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('question', django_bleach.models.BleachField(max_length=150)),
            ],
            options={
                'db_table': 'security_questions',
            },
        ),
        migrations.CreateModel(
            name='SecurityAnswer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('answer', django_bleach.models.BleachField(max_length=100)),
                ('user', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='user_security_answers', to=settings.AUTH_USER_MODEL)),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_security_questions.securityquestion', verbose_name='Security Question')),
            ],
            options={
                'db_table': 'security_answer',
                'unique_together': {('user', 'question')},
            },
        ),
    ]
